var grid, data;

// Handle formatting of a clickable item
function AnchorFormatter(row, cell, value, columnDef, dataContext) {
    var url = dataContext['url'];
    return '<a href="' + url + '">' + value + '</a>'
}

var columns = [
    {
        id:"#",
        name:"",
        width:50,
        behavior:"selectAndMove",
        selectable:false,
        resizable:false,
        cssClass:"cell-reorder dnd"
    },
    {
        id:"title",
        name:"Title",
        field:"title",
        width:350,
        formatter:AnchorFormatter
    },
    {
        id:"name",
        name:"Name",
        field:"name",
        width:300,
        cssClass:"cell-title",
        editor:Slick.Editors.Text,
        validator:requiredFieldValidator
    }
];

var options = {
    editable:true,
    enableRowReordering:true,
    enableCellNavigation:true,
    forceFitColumns:true,
    autoEdit:false,
};

function requiredFieldValidator(value) {
    if (value == null || value == undefined || !value.length) {
        return {valid:false, msg:"This is a required field"};
    } else {
        return {valid:true, msg:null};
    }
}

function initialize_grid() {
    grid = new Slick.Grid(".sd_grid", data, columns, options);

    grid.setSelectionModel(new Slick.RowSelectionModel());

    var moveRowsPlugin = new Slick.RowMoveManager();

    moveRowsPlugin.onBeforeMoveRows.subscribe(function (e, data) {
        for (var i = 0; i < data.rows.length; i++) {
            // no point in moving before or after itself
            if (data.rows[i] == data.insertBefore || data.rows[i] == data.insertBefore - 1) {
                e.stopPropagation();
                return false;
            }
        }
        return true;
    });

    moveRowsPlugin.onMoveRows.subscribe(function (e, args) {
        var extractedRows = [], left, right;
        var rows = args.rows;
        var insertBefore = args.insertBefore;
        left = data.slice(0, insertBefore);
        right = data.slice(insertBefore, data.length);

        rows.sort(function (a, b) {
            return a - b;
        });

        for (var i = 0; i < rows.length; i++) {
            extractedRows.push(data[rows[i]]);
        }

        rows.reverse();

        for (var i = 0; i < rows.length; i++) {
            var row = rows[i];
            if (row < insertBefore) {
                left.splice(row, 1);
            } else {
                right.splice(row - insertBefore, 1);
            }
        }

        data = left.concat(extractedRows.concat(right));

        var selectedRows = [];
        for (var i = 0; i < rows.length; i++)
            selectedRows.push(left.length + i);

        grid.resetActiveCell();
        grid.setData(data);
        grid.setSelectedRows(selectedRows);
        grid.render();
    });

    grid.registerPlugin(moveRowsPlugin);
    $.drop({mode:"mouse"});
}

$(function () {
    var json_url = $('.sd_grid').data('json-url');
    $.getJSON(json_url)
        .success(function (new_data) {
                     data = new_data;
                     initialize_grid();
                 });
})