from random import (
    choice,
    randint,
    shuffle
    )

from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_config

from substanced.sdi import mgmt_view
from substanced.interfaces import IFolder

random_words = (
    'Document', 'Paper', 'Study', 'Journal', 'Manual', 'Treatise',
    'File', 'Gray', 'Blue', 'Orange', 'Red', 'First', 'Last'
    )

@mgmt_view(
    context=IFolder,
    name='grid_view',
    tab_title='Grid',
    permission='sdi.add-content',
    renderer='templates/grid_view.pt'
)
def grid_view(context, request):
    # Filter only the folder items that have a "title"
    json_url = request.resource_url(context, 'grid_items.json')
    return dict(json_url=json_url)


@view_config(context=IFolder, name='grid_items.json', renderer='json')
def grid_items(context, request):
    items = []
    for item in context.values():
        if getattr(item, 'title', False):
            items.append(
                dict(
                    title=item.title,
                    name=item.__name__,
                    url=request.mgmt_path(item, '@@manage_main')
                )
            )
    return items


@view_config(context=IFolder, name="random_documents")
def random_document(context, request):
    registry = request.registry

    for i in range(100):
        # Make a title
        v = [choice(random_words), str(randint(10000,99999)),
             choice(random_words)]
        shuffle(v)
        title = v[0] + ' ' + v[1] + ' ' + v[2]

        appstruct = dict(title=title)
        document = registry.content.create('Document', **appstruct)
        # Simple algorithm to construct a name from the title
        name = appstruct['title'].lower().replace(' ', '-')
        context[name] = document

    return HTTPFound(request.mgmt_path(context, '@@grid_view'))